from decimal import Decimal
from django.conf import settings
from django.core.signing import Signer
from django.core.validators import MinValueValidator
from django.db import models
from django.dispatch import receiver
from django.urls import reverse

from paypal.standard.models import ST_PP_COMPLETED
from paypal.standard.ipn.signals import valid_ipn_received


class Invoice(models.Model):
    STATUS_CHOICES = (
        ('new', 'Invoiced'),
        ('pending', 'Payment pending'),
        ('paid', 'Payment received'),
        ('canceled', 'Invoice canceled'),
    )

    reference_number = models.CharField(max_length=128, unique=True,
                                        null=False, blank=False)
    status = models.CharField(max_length=128, choices=STATUS_CHOICES,
                              default='new')
    date = models.DateField()

    recipient = models.ForeignKey(settings.AUTH_USER_MODEL,
                                  related_name='invoices',
                                  on_delete=models.PROTECT)
    invoiced_entity = models.CharField(max_length=128, blank=True)
    billing_address = models.TextField()
    compound = models.BooleanField(default=False)

    transaction_id = models.CharField(max_length=128, null=False, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    last_update = models.DateTimeField(auto_now=True)

    signer = Signer(sep=':', salt='invoices.Invoice')

    @property
    def total(self):
        return sum(line.total for line in self.lines.all())

    @property
    def total_local(self):
        return round(self.total * settings.DEBCONF_LOCAL_CURRENCY_RATE, 2)

    def paypal_data(self):
        data = settings.PAYPAL_DATA.copy()

        data['cmd'] = '_cart'
        data['invoice'] = settings.INVOICE_PREFIX + self.reference_number
        data['upload'] = '1'

        i = 1
        for line in self.lines.all():
            if line.quantity * line.unit_price < 0:
                discount = data.get('discount_amount_cart', 0)
                discount += - (line.quantity * line.unit_price)
                data['discount_amount_cart'] = discount
                continue
            data['item_name_%d' % i] = line.description
            data['quantity_%d' % i] = line.quantity
            data['amount_%d' % i] = line.unit_price
            i = i + 1

        return data

    def text_details(self):
        header = ('Reference', 'Description', 'Qty', 'Unit', 'Total')
        footer = ('', '', '', 'Total', str(self.total))
        lines = [
            (line.reference, line.description, line.quantity, line.unit_price,
             line.total)
            for line in self.lines.all()
        ]
        all_lines = (
            [
                header,
                ('', '', '', '', ''),
            ]
            + lines
            + [
                ('', '', '', '-------', '-------'),
                footer,
            ]
        )

        col_width = [max(len(str(x)) for x in col) for col in zip(*all_lines)]
        formats = ['{:{}}'] * 3 + ['{:>{}}'] * 2
        return "\n".join(
            ("| " + " | ".join(formats[i].format(x, col_width[i])
                               for i, x in enumerate(line)) + " |")
            for line in all_lines
        )

    def save(self, *args, **kwargs):
        # generate reference number on save
        if not self.reference_number:
            year = str(self.date.year)
            last_invoice = Invoice.objects.filter(
                reference_number__startswith=year
            ).order_by('-reference_number').first()

            if last_invoice:
                year, seqnum = last_invoice.reference_number.split('-')
                seqnum = int(seqnum, 10) + 1
            else:
                seqnum = 1

            self.reference_number = '%s-%05d' % (year, seqnum)

        return super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse(
            'invoices:display',
            kwargs={'reference_number': self.reference_number})

    def get_signed_url(self):
        signed_reference = self.signer.sign(self.reference_number)
        return reverse(
            'invoices:display',
            kwargs={'reference_number': signed_reference})


@receiver(valid_ipn_received)
def update_invoice_on_ipn_received(sender, **kwargs):
    ipn = sender
    paypal_data = settings.PAYPAL_DATA
    if ipn.payment_status == ST_PP_COMPLETED:
        if ipn.receiver_email != paypal_data['business']:
            return
        if not ipn.invoice.startswith(settings.INVOICE_PREFIX):
            return

        invoice_number = ipn.invoice[len(settings.INVOICE_PREFIX):]
        invoice = Invoice.objects.get(reference_number=invoice_number)
        invoice.status = 'paid'
        invoice.transaction_id = ipn.txn_id
        invoice.save()

        if invoice.compound:
            for line in invoice.lines.all():
                reference = line.reference.split('#', 1)[1]
                child_invoice = Invoice.objects.get(reference_number=reference)
                child_invoice.status = 'paid'
                child_invoice.transaction_id = ipn.txn_id
                child_invoice.save()


class InvoiceLine(models.Model):
    invoice = models.ForeignKey(Invoice, related_name='lines',
                                on_delete=models.CASCADE)
    line_order = models.IntegerField()
    reference = models.CharField(max_length=32)
    description = models.CharField(max_length=1024)
    unit_price = models.DecimalField(max_digits=8, decimal_places=2)
    quantity = models.IntegerField(validators=[MinValueValidator(1)])

    @property
    def total(self):
        return self.unit_price * self.quantity

    class Meta:
        unique_together = ('invoice', 'line_order')
        ordering = ('invoice', 'line_order')

    def __str__(self):
        return 'InvoiceLine(%s [%s] %d @ %.02f = %.02f)' % (
            self.reference,
            self.description,
            self.quantity,
            self.unit_price,
            self.total,
        )



from datetime import datetime

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.template import Context, Template

from register.models import Attendee
from wafer.users.models import UserProfile


SUBJECT = '{{ WAFER_CONFERENCE_NAME }}: Registration Reminder'
TXT = '''Hi,

{% if incomplete_registration %}We see that you started a registration on https://{{ WAFER_CONFERENCE_DOMAIN }}
but have not yet completed it.
{% else %}We see that you have created an account on https://{{ WAFER_CONFERENCE_DOMAIN }}
but have not yet actually registered to attend the conference.
{% endif %}
If you are intending to attend, you need to go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>
and fill out every page of the form.

When you are successfully registered, you'll receive a confirmation e-mail, and
see a big green "Registered" label on your profile.

{% if incomplete_registration %}If you no longer plan to attend {{ WAFER_CONFERENCE_NAME }}, go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/unregister/>
and click the "Unregister" button.

{% endif %}{% if before_bursary_deadline %}Bursary applications close on {{ DEBCONF_BURSARY_DEADLINE|date:"SHORT_DATE_FORMAT" }}.
If you are applying for a bursary, you need to complete your registration
before this date.

We recommend that you register as soon as possible.
{% else %}Unfortunately, bursary applications have already closed.

We recommend that you register as soon as possible, if you're planning to
attend at your own cost.
{% endif %}If you need to amend your registration later, you can.

Hope to see you in {{ DEBCONF_CITY }},

The {{ WAFER_CONFERENCE_NAME }} team
'''


class Command(BaseCommand):
    help = "Badger users who haven't registered for attendance"

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, dry_run, site):
        bursary_deadline = settings.DEBCONF_BURSARY_DEADLINE
        today = datetime.utcnow().date()
        try:
            attendee = user.attendee
            incomplete_registration = True
        except Attendee.DoesNotExist:
            incomplete_registration = False

        context = Context({
            'before_bursary_deadline': bursary_deadline >= today,
            'incomplete_registration': incomplete_registration,
            'DEBCONF_BURSARY_DEADLINE': bursary_deadline,
            'DEBCONF_CITY': settings.DEBCONF_CITY,
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for user in UserProfile.objects.all():
            if not user.is_registered():
                self.badger(user.user, dry_run, site)



from django.contrib import admin

from invoices.models import Invoice, InvoiceLine


class InvoiceLineInline(admin.TabularInline):
    model = InvoiceLine


class InvoiceAdmin(admin.ModelAdmin):
    list_display = ('reference_number', 'recipient', 'total', 'date', 'status')
    list_filter = ('status',)
    search_fields = ('reference_number', 'recipient__username',
                     'recipient__first_name', 'recipient__last_name')
    inlines = [InvoiceLineInline]


admin.site.register(Invoice, InvoiceAdmin)



# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from invoices.prices import ACCOMM_INVOICE_INFO
from invoices.models import InvoiceLine

SUBJECT = u'[DebConf18]: Invoice cancelled due to price change'
TXT = u"""Hi,

The price of on-site accommodation at DebConf18 has decreased.  We have
accordingly cancelled your previous invoice [0].  Please visit the registration
wizard [1] again in order to generate a new invoice at the lower price.

[0]: https://debconf18.debconf.org/invoices/{reference_number}/
[1]: https://debconf18.debconf.org/register

Thanks for your understanding,

The DebConf registration team
"""


class Command(BaseCommand):
    help = 'Cancel pending overpriced accommodation invoices'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')

    def cancel(self, invoice, dry_run):
        if dry_run:
            print('I would cancel: {}'.format(invoice.reference_number))
            return
        invoice.status = 'canceled'
        invoice.save()

        attendee = invoice.recipient.attendee
        attendee.completed_register_steps = 9
        attendee.save()

        to = invoice.recipient.email
        body = TXT.format(reference_number=invoice.reference_number)
        email_message = EmailMultiAlternatives(SUBJECT, body, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        seen = set()
        for line in InvoiceLine.objects.filter(
                    invoice__status='new',
                    reference=ACCOMM_INVOICE_INFO['reference']
                ).exclude(unit_price=ACCOMM_INVOICE_INFO['unit_price']):
            invoice = line.invoice
            if invoice.reference_number in seen:
                continue
            seen.add(invoice.reference_number)

            self.cancel(invoice, dry_run)



from django.conf import settings

from register.forms.personal import PersonalInformationForm
from register.models.attendee import Attendee
from register.models.queue import Queue
from register.views.core import RegisterStep


class PersonalInformationView(RegisterStep):
    title = 'Personal Information'
    form_class = PersonalInformationForm

    def get_initial(self):
        user = self.request.user
        initial = {}

        attendee = user.attendee
        for field in attendee._meta.get_fields():
            if field.is_relation:
                continue
            initial[field.name] = getattr(attendee, field.name)

        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee  # We should never be creating, here
        user.attendee, created = Attendee.objects.update_or_create(
            user=user, defaults=data)

        if form.cleaned_data['pgp_fingerprints'] and settings.ISSUE_KSP_ID:
            queue, created = Queue.objects.get_or_create(
                name='PGP Keysigning')
            queue.slots.get_or_create(attendee=user.attendee)

        return super().form_valid(form)



from register.forms.misc import MiscForm
from register.models.attendee import Attendee
from register.views.core import RegisterStep


class MiscView(RegisterStep):
    title = 'Anything Else?'
    form_class = MiscForm

    def get_initial(self):
        initial = {
            'notes': self.request.user.attendee.notes
        }
        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee  # We should never be creating, here
        user.attendee, created = Attendee.objects.update_or_create(
            user=user, defaults=data)
        return super().form_valid(form)



from django import template
from django.conf import settings

register = template.Library()

debconf = {
    'LOCAL_CURRENCY': settings.DEBCONF_LOCAL_CURRENCY,
}

@register.simple_tag
def debconf_setting(key):
    return debconf[key]


# These colors were taken by picking the lightest colors from
# https://en.wikipedia.org/wiki/Web_colors#X11_color_names then shuffling the
# list
colors = [
    ('#FFE4E1', 'MistyRose'),
    ('#F0FFF0', 'Honeydew'),
    ('#00FFFF', 'Cyan'),
    ('#E0FFFF', 'LightCyan'),
    ('#00FF00', 'Lime'),
    ('#D2B48C', 'Tan'),
    ('#FFD700', 'Gold'),
    ('#D3D3D3', 'LightGray'),
    ('#B0C4DE', 'LightSteelBlue'),
    ('#E6E6FA', 'Lavender'),
    ('#00BFFF', 'DeepSkyBlue'),
    ('#F5DEB3', 'Wheat'),
    ('#90EE90', 'LightGreen'),
    ('#FFB6C1', 'LightPink'),
    ('#FFA07A', 'LightSalmon'),
    ('#AFEEEE', 'PaleTurquoise'),
    ('#DDA0DD', 'Plum'),
    ('#FFFFE0', 'LightYellow'),
    ('#F0E68C', 'Khaki'),
]


@register.simple_tag
def debconf_track_color(track):
    return colors[track.id % len(colors)][0]



# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.template import Context, engines

from invoices.models import Invoice


class Command(BaseCommand):
    help = 'Badger outstanding invoices'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),

    def badger(self, invoice, dry_run):
        to = invoice.recipient.email
        name = invoice.recipient.get_full_name()
        invoice_number = invoice.reference_number
        total = invoice.total

        if not total:
            return

        if dry_run:
            print('I would badger %s <%s> (%s: %s CAD)' %
                  (name, to, invoice_number, total))
            return

        ctx = Context({
            'name': name,
            'total': total,
            'invoice_number': invoice_number,
            'invoice_details': invoice.text_details(),
        })

        template = engines['django'].get_template(
            'invoices/outstanding_invoices-subject.txt')
        subject = template.render(ctx)

        template = engines['django'].get_template(
            'invoices/outstanding_invoices-body.txt')
        body = template.render(ctx)

        email_message = EmailMultiAlternatives(subject, body,
                                               to=["%s <%s>" % (name, to)])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for invoice in Invoice.objects.filter(status='new'):
            self.badger(invoice, dry_run)



import json
import logging

from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import EmailMultiAlternatives
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Model
from django.forms.models import model_to_dict
from django.template.loader import render_to_string

from bursary.models import Bursary
from invoices.prices import invoice_user
from register.forms.billing import BillingForm
from register.models import Attendee, Food
from register.views.core import RegisterStep


class BillingView(RegisterStep):
    title = 'Billing'
    template_name = 'register/page/billing.html'
    form_class = BillingForm

    def get_initial(self):
        attendee = self.request.user.attendee
        initial = {
            'billing_address': attendee.billing_address,
            'invoiced_entity': attendee.invoiced_entity,
        }
        return initial

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['attendee'] = self.request.user.attendee
        return kwargs

    def will_invoice(self):
        attendee = self.request.user.attendee
        if not attendee.billable():
            return False
        if attendee.paid():
            return False
        return True

    def will_reinvoice(self):
        user = self.request.user
        attendee = user.attendee
        if not attendee.new_invoices.exists():
            return False

        total = sum(invoice.total for invoice in attendee.new_invoices.all())

        invoice = invoice_user(user)
        return invoice['total'] != total

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        user = self.request.user
        attendee = user.attendee

        invoice = None
        if self.will_invoice():
            invoice = invoice_user(user)

        context.update({
            'attendee': attendee,
            'will_reinvoice': self.will_reinvoice(),
            'invoice': invoice,
            'DEBCONF_LOCAL_CURRENCY': settings.DEBCONF_LOCAL_CURRENCY,
            'DEBCONF_LOCAL_CURRENCY_SYMBOL':
                settings.DEBCONF_LOCAL_CURRENCY_SYMBOL,
        })
        return context

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee, created = Attendee.objects.update_or_create(
            user=user, defaults=data)

        if self.will_reinvoice():
            user.attendee.new_invoices.update(status='canceled')

        if self.will_invoice():
            invoice_user(user, save=True)

        self.confirm_registration()

        return super().form_valid(form)

    def get_email_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        user = self.request.user
        attendee = user.attendee

        try:
            bursary = user.bursary
        except Bursary.DoesNotExist:
            bursary = None

        try:
            food = user.attendee.food
        except ObjectDoesNotExist:
            food = None

        try:
            accomm = user.attendee.accomm
        except ObjectDoesNotExist:
            accomm = None

        try:
            child_care = user.attendee.child_care
        except ObjectDoesNotExist:
            child_care = None

        context = super().get_context_data(**kwargs)

        context.update({
            'accomm': accomm,
            'attendee': attendee,
            'bursary': bursary,
            'child_care': child_care,
            'food': food,
            'profile': user.userprofile,
            'user': user,
            'DEBCONF_CITY': settings.DEBCONF_CITY,
            'DEBCONF_DATES': settings.DEBCONF_DATES,
            'DEBCONF_SHOE_SIZES': settings.DEBCONF_SHOE_SIZES,
        })
        if attendee:
            context.update({
                'fee': Attendee.FEES[attendee.fee],
                'gender': Attendee.GENDERS[attendee.gender],
                't_shirt_size':
                    dict(settings.DEBCONF_T_SHIRT_SIZES)[attendee.t_shirt_size],
            })
        if bursary:
            context['bursary_need'] = Bursary.BURSARY_NEEDS.get(bursary.need)
        if food:
            context['diet'] = Food.DIETS[food.diet]

        return context

    def confirm_registration(self):
        fresh_registration = False
        user = self.request.user
        attendee = user.attendee
        if attendee.completed_register_steps <= self.register_step:
            fresh_registration = True
            attendee.completed_register_steps = self.register_step + 1
            attendee.save()

        context = self.get_email_context_data()

        self.send_registered_email(user, context, fresh_registration)
        self.log_registration(user, context, fresh_registration)

    def send_registered_email(self, user, context, fresh_registration):
        txt = render_to_string('register/email_received.txt',
                               context, request=self.request)
        to = user.email
        site = get_current_site(self.request)
        if fresh_registration:
            subject = '[{}] Registration received'
        else:
            subject = '[{}] Registration updated'
        subject = subject.format(site.name)
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def log_registration(self, user, context, fresh_registration):
        log = logging.getLogger('register')
        data = {}
        m2m = {
            'accomm': ['nights'],
            'food': ['meals'],
        }
        for model in ('accomm', 'attendee', 'bursary', 'child_care', 'food',
                      'profile', 'user'):
            if context[model]:
                data[model] = model_to_dict(context[model])
                if model in m2m:
                    for field in m2m[model]:
                        data[model][field] = [
                            meal.form_name for meal in data[model][field]
                        ]
            else:
                data[model] = {}
        data['user'].pop('password')

        log.info('User registered: user=%s updated=%s data=%s',
                 user.username, not fresh_registration,
                 json.dumps(data, cls=LogRegistrationJSONEncoder))


class LogRegistrationJSONEncoder(DjangoJSONEncoder):
    def default(self, o):
        if isinstance(o, Model):
            return str(o)
        return super().default(o)



from django.conf.urls import url

from invoices.views import InvoiceCancel, InvoiceCombine, InvoiceDisplay

app_name = 'invoices'
urlpatterns = [
    url(r'^combine/$', InvoiceCombine.as_view(), name='combine'),
    url(r'^(?P<reference_number>[^/]+)/$', InvoiceDisplay.as_view(),
        name='display'),
    url(r'^(?P<reference_number>[^/]+)/cancel/$', InvoiceCancel.as_view(),
        name='cancel'),
]



from datetime import timedelta
from django.utils import timezone


def get_start_end_for_scheduleitem(si):
    tz = timezone.get_default_timezone()
    if not si.slots.exists():
        raise ValueError('M2M not realized yet, come back later')
    start = timezone.make_aware(si.get_start_datetime(), tz)
    end = start + timedelta(minutes=si.get_duration_minutes())
    return (start, end)



# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from dc17.models.attendee import Attendee

SUBJECT = ('[ACTION REQUIRED]: Are you coming to DebConf17? '
           'Confirm your attendance *now*!')

TXT = """\
Dear %(name)s,

Our records indicate that you haven't confirmed your attendance at DebConf17.

The deadline to confirm is July 15 23:59 UTC. After this date, we cannot
guarantee any accommodation or food that you may have requested. Your badge
won't be printed and we probably won't have a t-shirt or swag to offer you
either.

If you want to confirm, simply log into the DebConf17 website, bring up your
profile through the upper right hand side menu and select "Update
Registration". On page 3 of that form, check "I confirm my attendance", then go
through the rest of the form verifying your data and Save.

If you know you're not going to be attending, please click the "Unregister"
link in your profile page.

Thank you,
The DebConf17 Registration Team
"""


class Command(BaseCommand):
    help = 'Badger users for confirmation'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),

    def badger(self, attendee, dry_run):
        to = attendee.user.email
        if dry_run:
            print('I would badger: %s <%s>' % (
                attendee.user.userprofile.display_name(),
                to,
            ))
            return

        body = TXT % {
            'name': attendee.user.userprofile.display_name(),
        }
        email_message = EmailMultiAlternatives(SUBJECT, body, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for attendee in Attendee.objects.filter(reconfirm=False):
            self.badger(attendee, dry_run)



from django import forms

from crispy_forms.helper import FormHelper


class MiscForm(forms.Form):
    notes = forms.CharField(
        label='Special Needs and Notes for the registration team',
        help_text='Anything else you need to describe. '
                  'The registration team will see this. '
                  'The bursaries team will not.',
        widget=forms.Textarea(attrs={'rows': 3}),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False



# -*- coding: utf-8 -*-

import decimal

from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from django.template import Context, Template

from dc17.models.bursary import Bursary

SUBJECT = "[Action needed] Reimbursement process for travel bursary recipients"

BODY_TEMPLATE = Template("""\
{% autoescape off %}
Dear {{object.user.get_full_name}},

You have been granted a travel bursary for DebConf, and now is the time you can
submit documentation for reimbursement. TL;DR: yes, this email is very long,
but all of it contains complete details about the process, so you will need to
read it in full to get travel money back!

Reimbursement for DebConf-related expenses will be processed by Software in the
Public Interest, for all Requestors including Europeans.

The amount you requested is {{object.travel_bursary}} USD. We have had some cancellations up until
the conference, so we have some budgetary leeway, but we ask you to keep your
reimbursement request within 10% of this amount (up to {{travel_bursary_max}} USD).

We're very sorry that the process took this long to materialize, but SPI has
been seeking long-term contracting help to process reimbursements faster, which
took some time to secure but is ready to go now. While this updated process
will take some time to gear up, we are confident that it will make
reimbursement processing run smoother for all SPI projects in the long run.

To process the hundreds of reimbursement requests efficiently and
expeditiously, SPI requires consistency. All requests must include:

 1. an SPI Reimbursement Request Form [1] with accurate banking information as
    SPI uses TransferWise [2]
 2. an Expense Report generated using XE's Travel Expenses Calculator [3], with
    a̲m̲o̲u̲n̲t̲s̲ ̲c̲o̲n̲v̲e̲r̲t̲e̲d̲ ̲t̲o̲ ̲U̲S̲D̲
 3. sufficient documentation substantiating the expense report
 4. a declaration of compliance

[1]: http://www.spi-inc.org/treasurer/SPI_reimbursement_request.pdf
[2]: https://transferwise.com
[3]: http://www.xe.com/travel-expenses-calculator/

Requestors must follow these steps:

Step 1: Prepare an SPI Reimbursement Request Form
 • download it from http://www.spi-inc.org/treasurer/SPI_reimbursement_request.pdf
 • e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲b̲a̲n̲k̲i̲n̲g̲ ̲i̲n̲f̲o̲r̲m̲a̲t̲i̲o̲n̲ ̲i̲s̲ ̲a̲c̲c̲u̲r̲a̲t̲e̲ ̲a̲s̲ ̲i̲n̲c̲o̲r̲r̲e̲c̲t̲ ̲d̲e̲t̲a̲i̲l̲s̲ ̲a̲r̲e̲ ̲a̲
   m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s̲
 • save as PDF

Step 2: Prepare an Expense Report.
 • use http://www.xe.com/travel-expenses-calculator/ to prepare an expense report
 • enter Your Name, which must match the name in the SPI Reimbursement Request Form
 • set Your Home Currency to USD as S̲P̲I̲ ̲w̲i̲l̲l̲ ̲p̲r̲o̲c̲e̲s̲s̲ ̲r̲e̲i̲m̲b̲u̲r̲s̲e̲m̲e̲n̲t̲s̲ ̲i̲n̲ ̲U̲S̲D̲
 • leave/set Credit Card @ 2%, Debit Card @ 5%, Foreign Cash @ 5%, Traveller's Cheque @ 2%
 • enter Receipt Details, one row per receipt
   • specify the correct date of the transactions
 • save as PDF

Step 3: Collect and order your Receipts.
 • collect your recipts in the SAME ORDER as the rows in the Expense Report
 • if paper receipts, scan them with a multi-function device, converting to PDF
 • save as PDF

Step 4: Prepare the Submission Package.
 • collect into a single PDF, the following:
   • from step 1, the SPI Reimbursement Request From
   • from step 2, the Expense Report
   • from step 3, the Ordered Receipts
 • the poppler-utils package includes the pdfunite utility
   • usage: `pdfunite step1.pdf step2.pdf step3a.pdf step3b.pdf ... step3n.pdf DebConf17ReimbursementRequest-{{short_full_name}}.pdf`
   • warning: ensure that you explicitly mention the destination filename
     (DebConf17ReimbursementRequest-{{short_full_name}}.pdf) otherwise
     step3n.pdf is overwritten
   • notice: please name the file exactly as shown, substituting your name as
     entered in the SPI Reimbursement Request Form, for example,
     DebConf17ReimbursementRequest-NicolasDandrimont.pdf
 • e̲n̲s̲u̲r̲e̲ ̲t̲h̲a̲t̲ ̲t̲h̲e̲ ̲e̲n̲t̲i̲r̲e̲ ̲S̲u̲b̲m̲i̲s̲s̲i̲o̲n̲ ̲P̲a̲c̲k̲a̲g̲e̲ ̲c̲a̲n̲ ̲b̲e̲ ̲e̲a̲s̲i̲l̲y̲ ̲u̲n̲d̲e̲r̲s̲t̲o̲o̲d̲ ̲a̲s̲ ̲p̲o̲o̲r̲
   q̲u̲a̲l̲i̲t̲y̲ ̲s̲u̲b̲m̲i̲s̲s̲i̲o̲n̲s̲ ̲a̲r̲e̲ ̲a̲ ̲m̲a̲j̲o̲r̲ ̲s̲o̲u̲r̲c̲e̲ ̲o̲f̲ ̲d̲e̲l̲a̲y̲s̲

Step 5: Email the Submission Package.
 • prepare an email having these attributes
   • from: you
   • To: debconf-reimbursements@rt.spi-inc.org
   • subject: DebConf17 Reimbursement Request for {{object.user.get_full_name}}
   • attachment: the Submission Package (DebConf17ReimbursementRequest-{{short_full_name}}.pdf) prepared in step 4
   • body: (the text below)
     By submitting this reimbursement request, I declare:
      • that I have accurately prepared an SPI Reimbursement Request Form,
      • that I have prepared an Expense Report using XE's Travel Expense Calculator,
      • that I have attached sufficient documentation substantiating my request,
      • that I seek reimbursement of expenses that are compliant to DebConf policies, and
      • that I have not sought nor will seek reimbursement of these expenses from any other source.

If you have any question, please be sure to contact us at bursaries@debconf.org
and we'll work your issues out.

Thanks for your cooperation!
--\u0020
The DebConf Bursaries team
{% endautoescape %}
""")


class Command(BaseCommand):
    help = 'Send an email to people whose bursary grant expires soon'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')

    def badger(self, bursary, dry_run):
        context = {
            'object': bursary,
            'user': bursary.user.username,
            'to': '%s <%s>' % (bursary.user.get_full_name(),
                               bursary.user.email),
            'short_full_name': bursary.user.get_full_name().replace(' ', '')
                                                           .replace('-', '')
                                                           .replace("'", '')
                                                           .replace('.', ''),
            'travel_bursary_max': int(bursary.travel_bursary
                                      * decimal.Decimal('1.1')),
        }

        if dry_run:
            print('I would badger {to} (max = {travel_bursary_max})'
                  .format(**context))
            return

        from_email = 'bursaries@debconf.org'
        subject = SUBJECT
        body = BODY_TEMPLATE.render(Context(context))

        msg = EmailMultiAlternatives(subject, body, to=[context['to']],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        to_badger = Bursary.objects.filter(
            request_travel=True,
            travel_status='accepted',
            user__attendee__reconfirm=True,
        )

        for bursary in to_badger:
            self.badger(bursary, dry_run)



# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from dc17.models.attendee import Attendee
from dc17.models.bursary import Bursary

SUBJECT = {
    True: 'Please check and update your DebConf17 registration data',
    False: 'Please confirm your DebConf17 attendance',
}

TXT = {}

TXT[True] = """\
Hi,

DebConf17 is now closer than ever. Our records indicate that you have confirmed
that you will be coming to DebConf, either through the registration form or
through the bursaries confirmation process.

Please make sure that your data is correct in the registration page at:
https://debconf17.debconf.org/accounts/profile/

We need accurate data before July 16th: your arrival and departure dates are
what helps us for accommodation and food planning. If it turns out that you are
unable to attend, you need to use the "Unregister" form on the aforementioned
profile page.
"""

TXT[False] = """\
Hi,

DebConf17 is now closer than ever. It is time for the organizers to get a list
of who will actually be attending the conference, and to collect payment for
those of you who registered as paying attendees.

The confirmation and invoicing process uses the registration wizard
linked on your profile at: https://debconf17.debconf.org/accounts/profile/

To confirm your attendance, you need to set final arrival and departure dates,
select the "My dates are final" entry and tick the "I confirm my attendance"
box. You need to go through the wizard until the end for your confirmation to
be valid! This makes it a perfect time to review your full registration data.
"""

INVOICING = """\
If you registered as a self-paying attendee, the confirmation process will
generate an invoice including your registration fee, as well as your choice of
accommodation and food for the duration of the conference. Once generated, the
invoice will show up on your profile page, and you will be able to settle it
online through the Debian PayPal account managed by SPI.
"""

DATES_TEMPLATE = """\
We have you registered as:
Arriving on {arrival}
Departing on {departure}
"""

FOOTER = """\
See you in Montréal,
The DebConf Registration team
"""


class Command(BaseCommand):
    help = 'Badger users for confirmation'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),

    def badger(self, attendee, dry_run):
        to = attendee.user.email
        confirmed = attendee.reconfirm
        self_paying = True

        try:
            bursary = Bursary.objects.get(user=attendee.user)
        except Bursary.DoesNotExist:
            pass
        else:
            if bursary.request_accommodation:
                self_paying = False

        action_needed = not confirmed

        if dry_run:
            print('I would badger: %s <%s> (%s, %s, %s)' % (
                attendee.user.userprofile.display_name(),
                to,
                'action needed' if action_needed else 'no action needed',
                'self paying' if self_paying else 'sponsored',
            ))
            return

        subject = (('[Action Needed] ' if action_needed else '')
                   + SUBJECT[confirmed])

        body_parts = []
        body_parts.append(TXT[confirmed])
        if self_paying:
            body_parts.append(INVOICING)
        body_parts.append(DATES_TEMPLATE.format(arrival=attendee.arrival,
                                                departure=attendee.departure))
        body_parts.append(FOOTER)

        body = '\n'.join(body_parts)

        email_message = EmailMultiAlternatives(subject, body, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for attendee in Attendee.objects.all():
            self.badger(attendee, dry_run)



from django.contrib import admin
from django.utils.html import format_html, format_html_join
from django.utils.http import urlquote

from bursary.models import Bursary, BursaryReferee


class BursaryRefereeInline(admin.TabularInline):
    model = BursaryReferee


class BursaryAdmin(admin.ModelAdmin):
    list_display = (
        'user',  'request_food', 'request_accommodation', 'request_travel',
        'need', 'travel_bursary', 'travel_from',
        'accommodation_status', 'accommodation_accept_before',
        'food_status', 'food_accept_before',
        'travel_status', 'travel_accept_before',
        'reimbursed_amount'
    )
    list_filter = ('accommodation_status', 'food_status', 'travel_status',
                   'request_food', 'request_accommodation', 'request_travel',
                   'accommodation_accept_before', 'food_accept_before',
                   'travel_accept_before')
    search_fields = ('user__username', 'user__first_name', 'user__last_name')
    inlines = (BursaryRefereeInline,)


admin.site.register(Bursary, BursaryAdmin)


class BursaryRefereeAdmin(admin.ModelAdmin):
    readonly_fields = (
        'full_name', 'email', 'presence', 'contributors_link',
        'bursary_request', 'bursary_need', 'bursary_travel',
        'bursary_travel_from', 'bursary_reason_contribution',
        'bursary_reason_plans', 'talk_proposals', 'bursary_reason_diversity',
    )

    def get_queryset(self, request):
        """Only list objects where the current user has to review"""
        qs = super(BursaryRefereeAdmin, self).get_queryset(request)

        if not request.user.has_perm('bursary.add_bursaryreferee'):
            qs = qs.filter(referee=request.user)

        return qs

    def has_change_permission(self, request, obj=None):
        """Only allow changing objects where the current user is reviewer"""
        hcp = super(BursaryRefereeAdmin, self).has_change_permission(request,
                                                                     obj)

        if (hcp and obj
                and not request.user.has_perm('bursary.add_bursaryreferee')):
            return obj.referee == request.user

        return hcp

    def is_admin(self, request):
        return request.user.has_perm('bursary.add_bursaryreferee')

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            ('Applicant informations', {
                'fields': ('full_name', 'email'),
            }),
            ('Bursary application', {
                'fields': (
                    'bursary_request', 'bursary_need',
                    'bursary_travel', 'bursary_travel_from',
                ),
            }),
            ('Contribution information', {
                'fields': (
                    'contributors_link', 'bursary_reason_contribution',
                    'presence', 'bursary_reason_plans', 'talk_proposals',
                    'contrib_score',
                ),
            }),
            ('Diversity and Inclusion', {
                'fields': (
                    'bursary_reason_diversity', 'outreach_score',
                )
            }),
            ('Notes', {'fields': ('notes', 'final')}),
        )

        # Display more fields for admin users; allows creation of referees by
        # hand
        if self.is_admin(request):
            fieldsets = (
                ('Bursary Referee admin', {
                    'fields': ('bursary', 'referee'),
                }),
            ) + fieldsets

        return fieldsets

    def get_list_display(self, request):
        if self.is_admin(request):
            return (
                'full_name', 'email', 'referee', 'final', 'bursary_request',
                'contrib_score', 'outreach_score', 'notes',
            )
        return (
            'full_name', 'email', 'final', 'bursary_request', 'contrib_score',
            'outreach_score', 'notes',
        )

    def get_list_filter(self, request):
        base_filters = ('final', 'contrib_score', 'outreach_score')
        if self.is_admin(request):
            return base_filters + ('referee',)
        return base_filters

    # Actions
    actions = ['make_referee_final']

    def get_actions(self, request):
        actions = super(BursaryRefereeAdmin, self).get_actions(request)
        if not self.is_admin(request):
            if 'delete_selected' in actions:
                del actions['delete_selected']

        return actions

    def make_referee_final(self, request, queryset):
        rows_updated = queryset.update(final=True)
        self.message_user(request, '%s evaluation%s finalized' % (
            rows_updated, 's' if rows_updated != 1 else '')
        )
    make_referee_final.short_description = "Mark selected evaluations as final"

    # Specific displays for evaluations
    def full_name(self, instance):
        return instance.bursary.user.get_full_name()
    full_name.admin_order_field = 'bursary__user__last_name'

    def email(self, instance):
        return instance.bursary.user.email
    email.admin_order_field = 'bursary__user__email'

    def contributors_link(self, instance):
        url = 'https://contributors.debian.org/contributors/mia/query?q=%s'
        email = instance.bursary.user.email
        return format_html(
            '<a href="{0}" target="_blank">{0}</a>',
            url % urlquote(email),
        )

    def presence(self, instance):
        arrival = instance.bursary.user.attendee.arrival
        departure = instance.bursary.user.attendee.departure

        return "%s to %s" % (arrival, departure)

    def bursary_request(self, instance):
        request = []
        if instance.bursary.request_food:
            request.append('food')
        if instance.bursary.request_accommodation:
            request.append('accommodation')
        if instance.bursary.request_travel:
            request.append('travel')
        return ','.join(request)

    def bursary_need(self, instance):
        return instance.bursary.get_need_display()
    bursary_need.short_description = 'Level of need (self-assessed)'

    def bursary_travel(self, instance):
        return instance.bursary.travel_bursary
    bursary_travel.short_description = 'Requested travel bursary amount (USD)'

    def bursary_travel_from(self, instance):
        return instance.bursary.travel_from
    bursary_travel_from.short_description = 'Traveling from'

    def bursary_reason_contribution(self, instance):
        return instance.bursary.reason_contribution
    bursary_reason_contribution.short_description = 'Contributions to Debian'

    def talk_proposals(self, instance):

        return format_html("<ul>{0}</ul>", format_html_join(
            '',
            '<li>{0} ({1}, <a href="{2}" target="_blank">details</a>)</li>',
            ((talk.title, talk.get_status_display(), talk.get_absolute_url())
             for talk in instance.bursary.user.talks.all())
        ))
    talk_proposals.short_description = 'Events proposed for the conference'

    def bursary_reason_plans(self, instance):
        return instance.bursary.reason_plans
    bursary_reason_plans.short_description = 'Plans for DebCamp or DebConf'

    def bursary_reason_diversity(self, instance):
        return instance.bursary.reason_diversity
    bursary_reason_diversity.short_description = (
        'Diversity and Inclusion criteria')


admin.site.register(BursaryReferee, BursaryRefereeAdmin)
